/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Routines shared by many pages
*/
#include "config.h"
#include "common.h"

/*
** Output a string with the following substitutions:
**
**     %T      The title of the current page.
**     %N      Project name
**     %V      CVSTrac version number
**     %%      The character '%'
*/
static void output_with_subst(const char *zText, const char *zTitle){
  int i;
  while( zText[0] ){
    for(i=0; zText[i] && zText[i]!='%'; i++){}
    if( i>0 ) cgi_append_content(zText, i);
    if( zText[i]==0 ) break;
    switch( zText[i+1] ){
      case 'T':
        zText += i+2;
        cgi_printf("%h", zTitle);
        break;
      case 'N':
        zText += i+2;
        cgi_printf("%h", g.zName);
        break;
      case 'V':
        zText += i+2;
        cgi_printf("%h", "@VERSION@");
        break;
      default:
        zText += i+1;
        break;
    }
  }
}

/*
** Generate an HTML header common to all web pages.  zTitle is the
** title for the page.  azLinks is an array of URI/Name pairs that
** are used to generate quick-links on the title bar.
*/
void common_header(const char *zTitle, const char **azLink){
  int i;
  int brk;
  char *zHeader = 0;

  zHeader = db_short_query("SELECT value FROM config WHERE name='header'");
  if( zHeader && zHeader[0] ){
    output_with_subst(zHeader, zTitle);
  }else{
    @ <html>
    @ <head>
    @ <title>%h(zTitle)</title>
    @ </head>
    @ <body bgcolor="white">
  }
  @ <table width="100%%" cellpadding=2 border=0>
  @ <tr><td bgcolor="%s(BORDER1)" class="border1">
  @ <table width="100%%" border=0 cellpadding=2 cellspacing=0>
  @ <tr bgcolor="%s(BG1)" class="bkgnd1">
  @ <td valign="top" align="left">
  @ <big><b>%h(g.zName) - %h(zTitle)</b></big><br>
  if( !g.isAnon ){
    @ <small>Logged in as <i>%h(g.zUser)</i></small>
  }else{
    @ <small>Not logged in</small>
  }
  @ </td>
  @ <td valign="bottom" align="right">
  if( azLink ){
    for(i=0; azLink[i*2]; i++){}
    brk = i<5 ? i+1 : i/2;
    brk = (brk-1)*2;
    for(i=0; azLink[i] && azLink[i+1]; i+=2){
      @ [<a href="%h(azLink[i])"><nobr>%h(azLink[i+1])</nobr></a>]&nbsp;
      if( i==brk ){
        @ <br>
      }
    }
  }
  if( i==0 ){
    @ &nbsp;
  }
  @ </td></tr></table>
  @ </td></tr></table>
}

/*
** Generate a common footer
*/
void common_footer(void){
  char *zFooter;
  zFooter = db_short_query("SELECT value FROM config WHERE name='footer'");
  if( zFooter && zFooter[0] ){
    output_with_subst(zFooter, "");
  }else{
    @ <p><small><small>
    @ <a href="about">CVSTrac version @VERSION@</a>
    @ </small></small></p>
    @ </body></html>
  }
}

/*
** Generate an about screen
**
** WEBPAGE: /about
*/
void common_about(void){
  static const char *azLink[] = { 
     "index", "Home", 
     0
  };
  login_check_credentials();
  common_header("About This Server", azLink);
  @ <p>This website is implemented using CVSTrac version @VERSION@.</p>
  @
  @ <p>CVSTrac implements a low-ceremony bug tracking system for small
  @ to medium-sized projects using CVS for version control.  For additional
  @ information, visit the CVSTrac homepage at</p>
  @ <blockquote>
  @ <a href="http://www.hwaci.com/sw/cvstrac/">
  @ http://www.hwaci.com/sw/cvstrac/
  @ </a>
  @ </blockquote>
  @
  @ <p>&copy; 2002 by <a href="mailto:drh@hwaci.com">D. Richard Hipp</a>.
  @ The CVSTrac server is released under the terms of the
  @ <a href="http://www.gnu.org/copyleft/gpl.html">GNU Public License</a>.</p>
  common_footer(); 
}
